@@ -94,7 +94,7 @@ if (getRversion() >= "3.1.0") {
 #'
 #' @param archive Optional character string giving the path of an archive
 #'   containing \code{targetFile}, or a vector giving a set of nested archives
-#'   (e.g., \code{c("xxx.tar", "inner.zip")}). If there is/are (an) inner
+#'   (e.g., \code{c("xxx.tar", "inner.zip", "inner.rar")}). If there is/are (an) inner
 #'   archive(s), but they are unknown, the function will try all until it finds
 #'   the \code{targetFile}. See table in \code{\link{preProcess}}.
 #'
@@ -123,7 +123,7 @@ if (getRversion() >= "3.1.0") {
 #'   \code{NULL} meaning do not search locally.
 #'
 #' @param fun Function or character string indicating the function to use to load
-#'   \code{targetFile} into an \code{R} object, e.g., in form wtih package name:
+#'   \code{targetFile} into an \code{R} object, e.g., in form with package name:
 #'   \code{"raster::raster"}.
 #'
 #' @param quick Logical. This is passed internally to \code{\link{Checksums}}
@@ -300,8 +300,12 @@ prepInputs <- function(targetFile = NULL, url = NULL, archive = NULL, alsoExtrac
         if (returnAsList)
           as.list(tmpEnv, all.names = TRUE)
       } else {
-        Cache(do.call, out$fun, append(list(asPath(out$targetFilePath)), args),
-              useCache = useCache)
+        mess <- capture.output(type = "message", obj <- Cache(do.call, out$fun, append(list(asPath(out$targetFilePath)), args),
+              useCache = useCache))
+        mess <- grep("No cacheRepo supplied", mess, invert = TRUE, value = TRUE)
+        if (length(mess) > 0)
+          message(mess)
+        obj
       }
     }
   } else {
@@ -352,9 +356,8 @@ extractFromArchive <- function(archive,
                                neededFiles = NULL, extractedArchives = NULL, checkSums = NULL,
                                needChecksums = 0, filesExtracted = character(),
                                checkSumFilePath = character(), quick = FALSE) {
-
   if (!is.null(archive)) {
-    if (!(any(c("zip", "tar", "tar.gz", "gz") %in% file_ext(archive)))) {
+    if (!(any(c("zip", "tar", "tar.gz", "gz", "rar") %in% file_ext(archive)))) {
       stop("Archives of type ", file_ext(archive), " are not currently supported. ",
            "Try extracting manually then placing extracted files in ", destinationPath)
     }
@@ -381,11 +384,8 @@ extractFromArchive <- function(archive,
         stop("No archive exists with filename: ", archive[1],
              ". Please pass an archive name to a path that exists")
       args <- list(archive[1], exdir = destinationPath[1])
-
       funWArgs <- .whichExtractFn(archive[1], args)
-
       filesInArchive <- .listFilesInArchive(archive)
-
       if (is.null(neededFiles)) {
         neededFiles <- basename(filesInArchive)
         result <- checkSums[checkSums$expectedFile %in% neededFiles, ]$result
@@ -397,50 +397,11 @@ extractFromArchive <- function(archive,
                                     checkSums = checkSums,
                                     checkSumFilePath = checkSumFilePath)
 
-      # checkSums <- if (isTRUE(file.exists(checkSumFilePath))) {
-      #   try(Checksums(
-      #     files = file.path(destinationPath, basename(neededFiles)),
-      #     checksumFile = checkSumFilePath,
-      #     path = destinationPath,
-      #     quickCheck = quick,
-      #     write = FALSE
-      #   ), silent = TRUE)
-      # } else {
-      #   needChecksums <- 1
-      #   checkSums <- .emptyChecksumsResult
-      # }
-
-      #if (is(checkSums, "try-error")) stop("checkSumFilePath is not a CHECKSUMS.txt file")
-
-      # join the neededFiles with the checkSums -- find out which are missing
-      # checkSumsDT <- data.table(checkSums)
-      # neededFilesDT <- data.table(neededFiles = basename(neededFiles))
-      # isOKDT <- checkSumsDT[neededFilesDT, on = c(expectedFile = "neededFiles")]
-      # isOKDT2 <- checkSumsDT[neededFilesDT, on = c(actualFile = "neededFiles")]
-      # # fill in any OKs from "actualFile" intot he isOKDT
-      # isOKDT[compareNA(isOKDT2$result, "OK"), "result"] <- "OK"
-      # isOK <- compareNA(isOKDT$result, "OK")
-
       isOK <- if (!is.null(checkSums)) {
         .compareChecksumsAndFiles(checkSums, neededFiles)
       } else {
         FALSE
       }
-      #basename(neededFiles) %in% checkSums$expectedFile
-      #isOK <-
-      #  checkSums[checkSums$expectedFile %in% basename(neededFiles) |
-      #              checkSums$actualFile %in% basename(neededFiles),]$result
-      #checkSums[compareNA(checkSums$expectedFile, basename(neededFiles)) |
-      #              compareNA(checkSums$actualFile, basename(neededFiles)),]$result
-      #isOK3 <- isOK$result[!is.na(isOK$result)] == "OK"
-
-      # filesInArchive <- funWArgs$fun(archive[1], list = TRUE)
-      #
-      # if ("Name" %in% names(filesInArchive)) {
-      #   # for zips, rm directories (length = 0)
-      #   filesInArchive <- filesInArchive[filesInArchive$Length != 0, ]$Name
-      # }
-
       # recheck, now that we have the whole file liast
       if (!(all(isOK)) ||
           NROW(result) == 0) {
@@ -448,22 +409,27 @@ extractFromArchive <- function(archive,
 
         # use binary addition -- 1 is new file, 2 is append
         if (needChecksums == 0) needChecksums <- 2
-
         if (length(archive) > 1) {
           filesExtracted <- c(filesExtracted,
                               .unzipOrUnTar(funWArgs$fun, funWArgs$args,
                                             files = basename(archive[2])))
           # recursion, removing one archive
-          extractedObjs <- extractFromArchive(archive[-1],
-                                              destinationPath = destinationPath,
-                                              neededFiles = neededFiles,
-                                              extractedArchives = extractedArchives,
-                                              checkSums = checkSums,
-                                              quick = quick,
-                                              needChecksums = needChecksums,
-                                              checkSumFilePath = checkSumFilePath,
-                                              filesExtracted = filesExtracted)
+          extractedObjs <- extractFromArchive(
+            archive[-1],
+            destinationPath = destinationPath,
+            neededFiles = neededFiles,
+            extractedArchives = extractedArchives,
+            checkSums = checkSums,
+            quick = quick,
+            needChecksums = needChecksums,
+            checkSumFilePath = checkSumFilePath,
+            filesExtracted = filesExtracted
+          )
         } else if (any(neededFiles %in% basename(filesInArchive)) || is.null(neededFiles)) {
+          possibleFolders <- filesInArchive[file_ext(filesInArchive) == ""]
+          if (length(possibleFolders)!=0){
+            filesInArchive <- setdiff(filesInArchive, possibleFolders)
+          }
           extractingTheseFiles <- paste(basename(filesInArchive[basename(filesInArchive) %in%
                                                                   neededFiles]),
                                         collapse = ", ")
@@ -472,16 +438,16 @@ extractFromArchive <- function(archive,
                                                                 collapse = ", "))
           message("From:", basename(archive[1]), "  \nExtracting\n ",
                   paste(collapse = "\n ", extractingTheseFiles))
-          filesExtracted <- c(filesExtracted,
-                              .unzipOrUnTar(funWArgs$fun, funWArgs$args,
-                                            files = filesInArchive[basename(filesInArchive) %in%
-                                                                     neededFiles]))
+            filesExtracted <- c(filesExtracted,
+                                .unzipOrUnTar(funWArgs$fun, funWArgs$args,
+                                              files = filesInArchive[basename(filesInArchive) %in%
+                                                                       neededFiles]))
         } else {
           # don't have a 2nd archive, and don't have our neededFiles file
-          isArchive <- grepl(file_ext(filesInArchive), pattern = "(zip|tar)", ignore.case = TRUE)
+          isArchive <- grepl(file_ext(filesInArchive), pattern = "(zip|tar|rar)", ignore.case = TRUE)
 
           if (any(isArchive)) {
-            arch <- filesInArchive[isArchive]
+            arch <- .basename(filesInArchive[isArchive])
             filesExtracted <- c(filesExtracted,
                                 .unzipOrUnTar(funWArgs$fun, funWArgs$args, files = arch))
 
@@ -523,6 +489,7 @@ extractFromArchive <- function(archive,
 #' @rdname guessAtTarget
 #' @name guessAtTarget
 #' @importFrom tools file_ext
+#' @importFrom utils unzip untar
 #' @inheritParams postProcess
 #' @param filesExtracted A character vector of all files that have been extracted (e.g.,
 #'                       from an archive)
@@ -548,11 +515,10 @@ extractFromArchive <- function(archive,
       #stop("Don't know what fun to use for loading targetFile. Please supply a 'fun'", call. = FALSE)
     }
   }
-
   if (is.null(targetFilePath)) {
     message("  targetFile was not specified. ", if (any(isShapefile)) {
       c(" Trying raster::shapefile on ", paste(possibleFiles[isShapefile], collapse = ", "), ".",
-        " If that is not correct, please specify different targetFile",
+        " If that is not correct, please specify a different targetFile",
         " and/or fun.")
     } else {
       c(" Trying ", fun,
@@ -574,12 +540,11 @@ extractFromArchive <- function(archive,
       } else {
         message("  Don't know which file to load. Please specify targetFile.")
       }
-
     }
     if (length(targetFilePath) > 1)  {
-      message("  More than one possible files to load, ", paste(targetFilePath, collapse = ", "),
-              ". Picking the first one. If not correct, specify a targetFile.")
-      targetFilePath <- targetFilePath[1]
+      message("  More than one possible files to load: ", paste(targetFilePath, collapse = ", "),
+              ". Picking the last one. If not correct, specify a targetFile.")
+      targetFilePath <- targetFilePath[length(targetFilePath)]
     } else {
       message("  Trying ", targetFilePath, " with ", fun, ".")
     }
@@ -599,6 +564,8 @@ extractFromArchive <- function(archive,
     args <- c(args, list(junkpaths = TRUE))
   } else if (ext == "tar") {
     fun <- untar
+  } else if (ext == "rar") {
+    fun <- "unrar"
   }
   out <- list(fun = fun, args = args)
   } else {
@@ -610,54 +577,103 @@ extractFromArchive <- function(archive,
 #' @keywords internal
 .unzipOrUnTar <- function(fun, args, files, overwrite = TRUE) {
   argList <- list(files = files)
-  isUnzip <- ("overwrite" %in% names(formals(fun)))
-  argList <- if (isUnzip) {
-    c(argList, overwrite = overwrite)
-  } else {
-    c(argList)
-  }
-  extractedFiles <- do.call(fun, c(args, argList))
-  worked <- if (isUnzip) {
-    all(file.path(args$exdir, basename(argList[[1]])) %in% extractedFiles)
-  } else {
-    isTRUE(extractedFiles == 0)
-  }
-  if (!isTRUE(worked)) {
-    message(
-      paste0(
-        "File unzipping do not appear to have worked properly.",
-        " Trying a system call of unzip..."
+  if (is.character(fun)) {
+    message(paste0("The archive is a .rar file. preProcess will try a system call of 'unrar'."))
+    hasUnrar <- .testForUnrar()
+    tempDir <- file.path(args$exdir, "extractedFiles") %>%
+      checkPath(create = TRUE)
+    if (grepl(x = hasUnrar, pattern = "7z")) {
+      system(
+        paste0("\"", hasUnrar, "\"", " e -o",
+               tempDir, " ",
+               args[[1]]),
+        wait = TRUE,
+        invisible = TRUE,
+        show.output.on.console = FALSE
       )
-    )
-    wd <- getwd()
-    tempDir <- file.path(args$exdir, "extractedFiles")
-    dir.create(tempDir, showWarnings = FALSE)
-    setwd(tempDir)
-    system2("unzip",
-            args = file.path(wd, args[[1]]),
-            wait = TRUE,
-            stdout = NULL)
+    } else {
+      system(paste0("unrar x ",
+                    args[[1]], " ",
+                    tempDir),
+             wait = TRUE, ignore.stdout = TRUE)
+    }
     extractedFiles <-
-      list.files(path = getwd(),
+      list.files(path = tempDir,
                  # list of full paths of all extracted files!
                  recursive = TRUE,
                  include.dirs = TRUE)
-    invisible(lapply(
-      X = extractedFiles,
-      FUN = function(fileToMove) {
-        file.rename(from = file.path(tempDir, fileToMove),
-                    to = file.path(args$exdir, fileToMove))
+    internalFolders <- extractedFiles[file_ext(extractedFiles) == ""]
+    extractedFiles <- setdiff(x = extractedFiles, y = internalFolders)
+    if (length(extractedFiles)==0) {
+      stop("preProcess could not extract the files from the archive ", basename(args[[1]]),".",
+           "Please try to extract it manually to the destinationPath")
+    } else {
+      invisible(lapply(
+        X = extractedFiles,
+        FUN = function(fileToMove) {
+          file.rename(from = file.path(tempDir, fileToMove),
+                      to = file.path(dirname(tempDir), basename(fileToMove)))
+        }
+      ))
+      unlink(file.path(args$exdir, "extractedFiles"), recursive = TRUE)
+      extractedFiles <- basename(extractedFiles)
+    }
+  } else {
+    isUnzip <- ("overwrite" %in% names(formals(fun)))
+    argList <- if (isUnzip) {
+      c(argList, overwrite = overwrite)
+    } else {
+      c(argList)
+    }
+    extractedFiles <- do.call(fun, c(args, argList))
+    worked <- if (isUnzip) {
+      all(normPath(file.path(args$exdir, basename(argList[[1]]))) %in% normPath(extractedFiles))
+    } else {
+      isTRUE(extractedFiles == 0)
+    }
+    if (!isTRUE(worked)) {
+      message(
+        paste0(
+          "File unzipping do not appear to have worked properly.",
+          " Trying a system call of unzip..."
+        )
+      )
+      tempDir <- file.path(args$exdir, "extractedFiles") %>%
+        checkPath(create = TRUE)
+      if (file.exists(args[[1]])){
+        pathToFile <-  normPath(args[[1]])
+      } else {
+        if (file.exists(file.path(args$exdir, args[[1]]))){
+          pathToFile <-  normPath(file.path(args$exdir, args[[1]]))
+        } else {
+          stop("prepInputs cannot find the file ", basename(args[[1]]),
+               ". Maybe the file was moved during unzipping?")
+        }
       }
-    ))
-    extractedFiles <- file.path(wd, extractedFiles)
-    unlink(file.path(args$exdir, "extractedFiles"), recursive = TRUE)
-    setwd(wd)
-    rm(wd)
+      system2("unzip",
+              args = paste0(pathToFile," -d ",tempDir),
+              wait = TRUE,
+              stdout = NULL)
+      extractedFiles <-
+        list.files(path = tempDir,
+                   # list of full paths of all extracted files!
+                   recursive = TRUE,
+                   include.dirs = TRUE)
+      invisible(lapply(
+        X = extractedFiles,
+        FUN = function(fileToMove) {
+          file.rename(from = file.path(tempDir, fileToMove),
+                      to = file.path(args$exdir, fileToMove))
+        }
+      ))
+      extractedFiles <- file.path(args$exdir, extractedFiles)
+      unlink(file.path(args$exdir, "extractedFiles"), recursive = TRUE)
+    }
+    if (!isUnzip) {
+      extractedFiles <- files
+    }
   }
-  if (!isUnzip) {
-    extractedFiles <- files
-  }
-  extractedFiles
+  return(extractedFiles)
 }
 
 #' @keywords internal
@@ -684,7 +700,7 @@ extractFromArchive <- function(archive,
 .isArchive <- function(filename) {
   if (!is.null(filename)) {
     filename <- if (length(filename)) {
-      isArchive <- file_ext(filename) %in% c("zip", "tar")
+      isArchive <- file_ext(filename) %in% c("zip", "tar", "rar")
       if (any(isArchive)) {
         filename[isArchive]
       } else {
@@ -727,10 +743,12 @@ appendChecksumsTable <- function(checkSumFilePath, filesToChecksum,
   message(messStart, "checksums to CHECKSUMS.txt. If you see this message repeatedly,\n",
           "  you can specify targetFile (and optionally alsoExtract) so it knows\n",
           "  what to look for.")
+  csf <- if (append) tempfile(fileext = ".TXT") else checkSumFilePath
   capture.output(
     type = "message",
-    currentFiles <- Checksums(path = destinationPath, write = TRUE,
-                              files = file.path(destinationPath, filesToChecksum))
+    currentFiles <- Checksums(path = destinationPath, write = TRUE, #write = !append || NROW(nonCurrentFiles) == 0,
+                              files = file.path(destinationPath, filesToChecksum),
+                              checksumFile = csf)
   )
   if (append) { # a checksums file already existed, need to keep some of it
     currentFilesToRbind <- data.table::as.data.table(currentFiles)
@@ -799,17 +817,47 @@ appendChecksumsTable <- function(checkSumFilePath, filesToChecksum,
 #' @keywords internal
 #' @rdname listFilesInArchive
 .listFilesInArchive <- function(archive) {
+  if (length(archive)>0 && tools::file_ext(archive[1]) == "rar"){
+    hasUnrar <- .testForUnrar()
+  }
   funWArgs <- .whichExtractFn(archive[1], NULL)
   filesInArchive <- NULL
   if (!is.null(funWArgs$fun)) {
     if (file.exists(archive[1])) {
-      filesInArchive <- funWArgs$fun(archive[1], list = TRUE)
-      if ("Name" %in% names(filesInArchive)) {
-        # for zips, rm directories (length = 0)
-        filesInArchive <-
-          filesInArchive[filesInArchive$Length != 0,]$Name
-      } else { # untar
-        filesInArchive
+      if (!file_ext(archive[1]) == "rar") {
+        filesInArchive <- funWArgs$fun(archive[1], list = TRUE)
+        if ("Name" %in% names(filesInArchive)) {
+          # for zips, rm directories (length = 0)
+          filesInArchive <-
+            filesInArchive[filesInArchive$Length != 0,]$Name
+        } else { # untar
+          filesInArchive
+        }
+      } else {
+        # On Windows
+        if (grepl(x = hasUnrar, pattern = "7z")) {
+          warn <- testthat::capture_warnings({
+            filesOutput <- system(paste0("\"", hasUnrar, "\"", " l ", archive[1]),
+                                  show.output.on.console = FALSE, intern = TRUE)
+          })
+        } else {
+          # On Linux/MacOS
+          filesOutput <- system(paste0("unrar l ", archive[1]), intern = TRUE)
+        }
+        if (exists("warn") && isTRUE(any(grepl("had status 2", warn))))
+          stop(warn)
+        if (isTRUE(any(grepl("Can not open the file as archive", filesOutput))))
+          stop("rar is defective")
+        filesInBetween <- grep(pattern = "----", filesOutput)
+        filesLines <- filesOutput[(min(filesInBetween)+1):(max(filesInBetween)-1)]
+        filesInArchive <- unlist(lapply(X = seq_along(filesLines), FUN = function(line){
+          fullString <- unlist(strsplit(filesLines[[line]], split = " "))
+          return(fullString[length(fullString)])
+        })
+        )
+        if (length(filesInArchive)==0) {
+          stop("preProcess could not find any files in the archive ", archive)
+        }
       }
     }
   }
@@ -826,3 +874,80 @@ appendChecksumsTable <- function(checkSumFilePath, filesToChecksum,
   isOK <- compareNA(isOKDT$result, "OK")
   isOK
 }
+
+#' Tests if unrar or 7zip exist
+#'
+#' @return
+#' unrar or 7zip path if exist, or NULL
+#'
+#' @author Tati Micheletti
+#'
+#' @keywords internal
+#' @rdname unrarExists
+#' @name unrarExists
+.unrarExists <- function() {
+    hasUnrar <- ""
+    hasUnrar <- Sys.which("unrar")
+    if (hasUnrar == "") {
+      hasUnrar <- Sys.which("7z.exe")
+      if (hasUnrar == "") {
+        message("prepInputs is looking for 'unrar' or '7z' in your system...")
+        hasUnrar <- list.files("C:/Program Files",
+                               pattern = "unrar.exe|7z.exe",
+                               recursive = TRUE,
+                               full.names = TRUE)
+        if (hasUnrar == "" || length(hasUnrar) == 0) {
+          hasUnrar <- list.files(dirname(Sys.getenv("SystemRoot")),
+                                 pattern = "unrar.exe|7z.exe",
+                                 recursive = TRUE,
+                                 full.names = TRUE)
+          if (hasUnrar == "" || length(hasUnrar) == 0) {
+            hasUnrar <- NULL
+          } else {
+            warning("The extracting software was found in an unusual location: ", hasUnrar, ".",
+                    "If you receive an error when extracting the archive, please install '7zip' or 'unrar'",
+                    " in 'Program Files' folder")
+          }
+        }
+        hasUnrar <- hasUnrar[1]
+      }
+    }
+  return(hasUnrar)
+}
+
+#' Returns unrar path and creates a shortcut as .unrarPath
+#' Was not incorporated in previous function so it can be
+#' used in the tests
+#'
+#' @return
+#' unrar or 7zip path if exist, and assign it to .unrarPath
+#' Stops and advise user to install it if unrar doesn't exist
+#'
+#' @author Tati Micheletti
+#'
+#' @keywords internal
+#' @rdname testForUnrar
+#' @name testForUnrar
+.testForUnrar <- function(){
+  if (!is.null(.unrarPath)) {
+    hasUnrar  <- .unrarPath
+  } else {
+    # Find the path to unrar and assign to a package-stored object
+    usrTg <- paste(sample(x = LETTERS, size = 15), collapse = "")
+    hasUnrar <- Cache(.unrarExists, userTags = usrTg) # Cache for project-level persistence
+    utils::assignInMyNamespace(".unrarPath", hasUnrar) # assign in namespace for package
+  }
+  if (is.null(hasUnrar)) {
+    clearCache(userTags = usrTg, ask = FALSE)
+    stop(
+      "prepInputs did not find '7-Zip' nor 'unrar' installed.",
+      " Please install it before running prepInputs for a '.rar' archive"
+    )
+  }
+  return(hasUnrar)
+}
+
+#' The known path for unrar or 7z
+#' @rdname unrarPath
+#' @name unrarPath
+.unrarPath <- NULL
